import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import seaborn as sb
import os
import warnings
warnings.filterwarnings('ignore')
import tensorflow as tf
from tensorflow.keras.layers.experimental.preprocessing import RandomFlip
from tensorflow.keras.layers import DepthwiseConv2D,Layer
from tensorflow.keras.preprocessing import image_dataset_from_directory


import keras

from tensorflow.keras import backend as K
from tensorflow.keras.models import Sequential, Model,load_model
from tensorflow.keras.callbacks import EarlyStopping,ModelCheckpoint
from tensorflow.keras.layers import Dropout,ReLU,Layer, Input, add, Dense, Activation, ZeroPadding2D, BatchNormalization, Flatten, Conv2D, AveragePooling2D, MaxPooling2D, GlobalMaxPooling2D,MaxPool2D
from tensorflow.keras.preprocessing import image
from tensorflow.keras.initializers import glorot_uniform

# Model Checks
from sklearn.metrics import accuracy_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score
from sklearn.metrics import roc_auc_score


import matplotlib.pyplot as plt 
from sklearn.preprocessing import LabelBinarizer
from sklearn.metrics import roc_curve, auc, roc_auc_score
from itertools import cycle

#Model Graphs
from sklearn.metrics import confusion_matrix
from sklearn.metrics import ConfusionMatrixDisplay
from keras.layers import Activation, Lambda,GlobalAveragePooling2D,concatenate
import math
from tensorflow.keras.layers import DepthwiseConv2D
from keras.layers import SeparableConv2D,Add,GlobalAvgPool2D
from keras.layers import Resizing
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers
import tensorflow_addons as tfa

def DualPathNetwork(input_shape=None,
                    in_filters=64,
                    depth=[3, 4, 20, 3],
                    filter_increment=[16, 32, 24, 128],
                    cardinality=32,
                    width=3,
                    input_tensor=None,
                    classes=1000):

    if input_tensor is None:
        img_input = Input(shape=(128,128,3))

    x = DPN(classes, in_filters, filter_increment, depth, cardinality, width)(img_input)

    model = Model(img_input, x, name='dpn')

    # load weights

    return model

###################################################

def DPN92(input_shape=None,
          input_tensor=None,
          classes=1000):
    
    return DualPathNetwork(classes=classes)

#####################################################

class BasicBnConv(Layer):
    def __init__(self, in_filters):
        super(BasicBnConv, self).__init__()
        self.conv = Conv2D(in_filters, (7, 7), padding='same', use_bias=False, strides=(2, 2))
        self.bn = BatchNormalization()
        self.relu = ReLU()
        self.maxpool = MaxPooling2D((3, 3), strides=(2, 2), padding='same')

    def call(self, inputs):
        x = self.conv(inputs)
        x = self.bn(x)
        x = self.relu(x)
        x = self.maxpool(x)
        
        return x
    
###############################################################


class ActBnConv(Layer):
    def __init__(self, filters, cardinality=1, kernel=(3, 3), stride=(1, 1)):
        super(ActBnConv, self).__init__()
        self.conv = Conv2D(filters, kernel, padding='same', use_bias=False,  strides=stride, groups=cardinality)
        self.bn = BatchNormalization()
        self.relu = ReLU()

    def call(self, inputs):
        x = self.conv(inputs)
        x = self.bn(x)
        x = self.relu(x)
        
        return x

################################################################

class DualPathBlock(Layer):
    def __init__(self, filters_1x1_a, filters_grouped_b, filters_1x1_c, filter_increment, cardinality, block_type='normal'):
        
        super(DualPathBlock, self).__init__()
        if block_type == 'projection':
            stride = (1, 1)
            self.projection = True
        elif block_type == 'downsample':
            stride = (2, 2)
            self.projection = True
        elif block_type == 'normal':
            stride = (1, 1)
            self.projection = False
            
        if self.projection:
            self.projection_path = ActBnConv(filters=filters_1x1_c + 2 * filter_increment, kernel=(1, 1), stride=stride)
            self.input_residual_path = Lambda(lambda z: z[:, :, :, :filters_1x1_c])
            self.input_dense_path = Lambda(lambda z: z[:, :, :, filters_1x1_c:])
            
        self.c_1x1_a = ActBnConv(filters=filters_1x1_a, kernel=(1, 1))
        self.c_3x3_b = ActBnConv(filters=filters_grouped_b, cardinality=cardinality, stride=stride)
        self.c_1x1_c = ActBnConv(filters=filters_1x1_c + filter_increment, kernel=(1, 1))
        
        self.output_residual_path = Lambda(lambda z: z[:, :, :, :filters_1x1_c])
        self.output_dense_path = Lambda(lambda z: z[:, :, :, filters_1x1_c:])

    def call(self, inputs):
        n_in = concatenate(inputs, axis=3) if isinstance(inputs, list) else inputs
        
        if self.projection:
            proj_path = self.projection_path(n_in)
            in_res_path = self.input_residual_path(proj_path)
            in_dense_path = self.input_dense_path(proj_path)
        else:
            in_res_path = inputs[0]
            in_dense_path = inputs[1]
            
        x = self.c_1x1_a(n_in)
        x = self.c_3x3_b(x)
        x = self.c_1x1_c(x)
        out_res_path = self.output_residual_path(x)
        out_dense_path = self.output_dense_path(x)
        
        residual_path = add([in_res_path, out_res_path])
        dense_path = concatenate([in_dense_path, out_dense_path], axis=3)

        return [residual_path, dense_path]
    
    
######################################################

class DPN(Layer):
    def __init__(self, nb_classes, in_filters, filter_increment, depth, cardinality=32, width=3):
        
        super(DPN, self).__init__()
        self.depth_list = depth #list(depth)
        self.base_filters = 256
        self.filter_list = filter_increment
        self.filters = int(cardinality * width)
        self.filter_inc = filter_increment[0]
        
        self.basic_bn_conv = BasicBnConv(in_filters)
        
        self.gap = GlobalAveragePooling2D()
        self.max = GlobalMaxPooling2D()
        self.flatten = Flatten()
        self.dense_1 = Dense(1000, use_bias=False)
        self.dense_2 = Dense(10, use_bias=False, activation='softmax')
        
    def dualBlocks(self, filters, base_filters, filter_inc, cardinality, block_type):
        block = DualPathBlock(filters_1x1_a=filters,
                              filters_grouped_b=filters,
                              filters_1x1_c=base_filters,
                              filter_increment=filter_inc,
                              cardinality=cardinality,
                              block_type=block_type)
        return block
            
        
    def call(self, inputs):
        
        deptht = self.depth_list
        base_filters = self.base_filters
        filter_list = self.filter_list
        filters = self.filters
        
        x = self.basic_bn_conv(inputs)
        
        filter_inc = self.filter_list[0]
        
        x = self.dualBlocks(filters, base_filters, filter_inc, 32, 'projection')(x)
        
        for i in range(deptht[0] - 1):
            x = self.dualBlocks(filters, base_filters, filter_inc, 32, 'normal')(x)

        for k in range(1, len(deptht)):
            filter_inc = self.filter_list[k]
            filters *= 2
            base_filters *= 2
            
            x = self.dualBlocks(filters, base_filters, filter_inc, 32, 'downsample')(x)

            for i in range(deptht[k] - 1):
                x = self.dualBlocks(filters, base_filters, filter_inc, 32, 'normal')(x)

        x = concatenate(x, axis=-1)


        x = self.gap(x)
        #max = self.max(x)
        #x = add([avg, max])
        #x = Lambda(lambda z: 0.5 * z)(x)
        #x = Lambda(lambda z: 0.5 * z)(avg)
        #x = self.flatten(x)
        x = self.dense_1(x)
        x = self.dense_2(x)


        return x






























import tensorflow
from PIL import Image
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import seaborn as sb
import os
import warnings
warnings.filterwarnings('ignore')
import tensorflow as tf
from tensorflow.keras.layers.experimental.preprocessing import RandomFlip
from tensorflow.keras.layers import DepthwiseConv2D,Layer
from tensorflow.keras.preprocessing import image_dataset_from_directory


import keras

from tensorflow.keras import backend as K
from tensorflow.keras.models import Sequential, Model,load_model
from tensorflow.keras.callbacks import EarlyStopping,ModelCheckpoint
from tensorflow.keras.layers import Dropout,ReLU,Layer, Input, add, Dense, Activation, ZeroPadding2D, BatchNormalization, Flatten, Conv2D, AveragePooling2D, MaxPooling2D, GlobalMaxPooling2D,MaxPool2D
from tensorflow.keras.preprocessing import image
from tensorflow.keras.initializers import glorot_uniform

# Model Checks
from sklearn.metrics import accuracy_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score
from sklearn.metrics import roc_auc_score


import matplotlib.pyplot as plt 
from sklearn.preprocessing import LabelBinarizer
from sklearn.metrics import roc_curve, auc, roc_auc_score
from itertools import cycle

#Model Graphs
from sklearn.metrics import confusion_matrix
from sklearn.metrics import ConfusionMatrixDisplay
from keras.layers import Activation, Lambda,GlobalAveragePooling2D,concatenate
import math
from tensorflow.keras.layers import DepthwiseConv2D
from keras.layers import SeparableConv2D,Add,GlobalAvgPool2D
from keras.layers import Resizing
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers
import tensorflow_addons as tfa
#############################################################
learning_rate = 0.001
weight_decay = 0.0001
batch_size = 256
num_epochs = 100
image_size = 224  # We'll resize input images to this size
patch_size = 16  # Size of the patches to be extract from the input images
num_patches = (image_size // patch_size) ** 2
projection_dim = 64
num_heads = 4
transformer_units = [
    projection_dim * 2,
    projection_dim,
]  # Size of the transformer layers
transformer_layers = 8
mlp_head_units = [2048, 1024] 
#############################################################

def mlp(x, hidden_units, dropout_rate):
    for units in hidden_units:
        x = layers.Dense(units, activation=tf.nn.gelu)(x)
        x = layers.Dropout(dropout_rate)(x)
    return x

#############################################################
data_augmentation = tensorflow.keras.Sequential(
    [
        layers.Normalization(),
        layers.Resizing(image_size, image_size),
        layers.RandomFlip("horizontal"),
        layers.RandomRotation(factor=0.02),
        layers.RandomZoom(
            height_factor=0.2, width_factor=0.2
        ),
    ],
    name="data_augmentation",
)
# Compute the mean and the variance of the training data for normalization.
#data_augmentation.layers[0].adapt(X_train)

#############################################################

class Patches(layers.Layer):
    def __init__(self, patch_size):
        super(Patches, self).__init__()
        self.patch_size = patch_size

    def call(self, images):
        batch_size = tf.shape(images)[0]
        patches = tf.image.extract_patches(
            images=images,
            sizes=[1, self.patch_size, self.patch_size, 1],
            strides=[1, self.patch_size, self.patch_size, 1],
            rates=[1, 1, 1, 1],
            padding="VALID",
        )
        patch_dims = patches.shape[-1]
        patches = tf.reshape(patches, [batch_size, -1, patch_dims])
        return patches
    

#############################################################



class PatchEncoder(layers.Layer):
    def __init__(self, num_patches, projection_dim):
        super(PatchEncoder, self).__init__()
        self.num_patches = num_patches
        self.projection = layers.Dense(units=projection_dim)
        self.position_embedding = layers.Embedding(
            input_dim=num_patches, output_dim=projection_dim
        )

    def call(self, patch):
        positions = tf.range(start=0, limit=self.num_patches, delta=1)
        encoded = self.projection(patch) + self.position_embedding(positions)
        return encoded

#############################################################

def visionTransformer(classes=1000):
    
    
    input_shape = (128, 128, 3)
    inputs = layers.Input(shape=input_shape)
    # Augment data.
    augmented = data_augmentation(inputs)
    # Create patches.
    patches = Patches(patch_size)(augmented)
    # Encode patches.
    encoded_patches = PatchEncoder(num_patches, projection_dim)(patches)

    # Create multiple layers of the Transformer block.
    for _ in range(transformer_layers):
        # Layer normalization 1.
        x1 = layers.LayerNormalization(epsilon=1e-6)(encoded_patches)
        # Create a multi-head attention layer.
        attention_output = layers.MultiHeadAttention(
            num_heads=num_heads, key_dim=projection_dim, dropout=0.1
        )(x1, x1)
        # Skip connection 1.
        x2 = layers.Add()([attention_output, encoded_patches])
        # Layer normalization 2.
        x3 = layers.LayerNormalization(epsilon=1e-6)(x2)
        # MLP.
        x3 = mlp(x3, hidden_units=transformer_units, dropout_rate=0.1)
        # Skip connection 2.
        encoded_patches = layers.Add()([x3, x2])

    # Create a [batch_size, projection_dim] tensor.
    representation = layers.LayerNormalization(epsilon=1e-6)(encoded_patches)
    representation = layers.Flatten()(representation)
    representation = layers.Dropout(0.5)(representation)
    # Add MLP.
    features = mlp(representation, hidden_units=mlp_head_units, dropout_rate=0.5)
    # Classify outputs.
    logits = layers.Dense(classes,activation = "softmax")(features)
    # Create the Keras model.
    model = tensorflow.keras.Model(inputs=inputs, outputs=logits)
    return model









import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import seaborn as sb
import os
import warnings
warnings.filterwarnings('ignore')
import tensorflow as tf
from tensorflow.keras.layers.experimental.preprocessing import RandomFlip
import tensorflow.keras.layers as tfl,DepthwiseConv2D
from tensorflow.keras.preprocessing import image_dataset_from_directory

import glob as gb
import cv2
import keras

from keras import backend as K
from keras.models import Sequential, Model,load_model
from keras.callbacks import EarlyStopping,ModelCheckpoint
from keras.layers import Dropout,ReLU,Layer, Input, add, Dense, Activation, ZeroPadding2D, BatchNormalization, Flatten, Conv2D, AveragePooling2D, MaxPooling2D, GlobalMaxPooling2D,MaxPool2D
from keras.preprocessing import image
from keras.initializers import glorot_uniform

# Model Checks
from sklearn.metrics import accuracy_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score
from sklearn.metrics import roc_auc_score


import matplotlib.pyplot as plt 
from sklearn.preprocessing import LabelBinarizer
from sklearn.metrics import roc_curve, auc, roc_auc_score
from itertools import cycle

#Model Graphs
from sklearn.metrics import confusion_matrix
from sklearn.metrics import ConfusionMatrixDisplay
from keras.layers import Activation, Lambda,GlobalAveragePooling2D,concatenate
import math
from tensorflow.keras.layers import DepthwiseConv2D
from keras.layers import SeparableConv2D,Add,GlobalAvgPool2D
from keras.layers import Resizing
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers
import tensorflow_addons as tfa








'''
import DLA34
import DPN92
import efficient_net_b0
import Xception
import InceptionV3
import ResNeXt
import VisionTransformer

from DLA34 import dla34
from DPN92 import DPN92
from efficient_net_b0 import efficient_net_b0
from Xception import Xception
from InceptionV3 import inceptionV3
from ResNeXt import resNeXt
from VisionTransformer import visionTransformer

'''
from .DLA34 import dla34
from .DPN92 import DPN92
from .efficient_net_b0 import efficient_net_b0
from .Xception import Xception
from .InceptionV3 import inceptionV3
from .ResNeXt import resNeXt
from .VisionTransformer import visionTransformer



def list_models():
    print("models list: dla34 \n DPN92 \n efficient_net_b0 \n Xception  \n inceptionV3 \n resNeXt \n visionTransformer")
        
    return 

def create_model(name, classes):
    if name == "dla34":
        model = dla34(10)
        return model
        
    elif name == "DPN92":
        model = DPN92(10)
        return model
        
    elif name == "efficient_net_b0":
        model = efficient_net_b0(10)
        return model
        
    elif name == "Xception":
        model = Xception(10)
        return model
        
    elif name == "inceptionV3":
        model = inceptionV3(10)
        return model
    
    elif name == "resNeXt":
        model = resNeXt(10)
        return model
    
    elif name == "visionTransformer":
        model = visionTransformer(10)
        return model
        
    print("models list: dla34 \n DPN92 \n efficient_net_b0 \n Xception  \n inceptionV3 \n resNeXt \n visionTransformer")
        
    return 

'''
#model = resNeXt(10)

model = create_model('resNeXt', 10)
model.summary()

list_models()
'''





def import_layers():
    
    from PIL import Image
    import matplotlib.pyplot as plt
    import pandas as pd
    import numpy as np
    import seaborn as sb
    import os
    import warnings
    warnings.filterwarnings('ignore')
    import tensorflow as tf
    from tensorflow.keras.layers.experimental.preprocessing import RandomFlip
    from tensorflow.keras.layers import DepthwiseConv2D,Layer
    from tensorflow.keras.preprocessing import image_dataset_from_directory
    
    
    import keras
    
    from tensorflow.keras import backend as K
    from tensorflow.keras.models import Sequential, Model,load_model
    from tensorflow.keras.callbacks import EarlyStopping,ModelCheckpoint
    from tensorflow.keras.layers import Dropout,ReLU,Layer, Input, add, Dense, Activation, ZeroPadding2D, BatchNormalization, Flatten, Conv2D, AveragePooling2D, MaxPooling2D, GlobalMaxPooling2D,MaxPool2D
    from tensorflow.keras.preprocessing import image
    from tensorflow.keras.initializers import glorot_uniform
    
    # Model Checks
    from sklearn.metrics import accuracy_score
    from sklearn.metrics import precision_score
    from sklearn.metrics import recall_score
    from sklearn.metrics import f1_score
    from sklearn.metrics import roc_auc_score
    
    
    import matplotlib.pyplot as plt 
    from sklearn.preprocessing import LabelBinarizer
    from sklearn.metrics import roc_curve, auc, roc_auc_score
    from itertools import cycle
    
    #Model Graphs
    from sklearn.metrics import confusion_matrix
    from sklearn.metrics import ConfusionMatrixDisplay
    from keras.layers import Activation, Lambda,GlobalAveragePooling2D,concatenate
    import math
    from tensorflow.keras.layers import DepthwiseConv2D
    from keras.layers import SeparableConv2D,Add,GlobalAvgPool2D
    from keras.layers import Resizing
    import numpy as np
    import tensorflow as tf
    from tensorflow import keras
    from tensorflow.keras import layers
    import tensorflow_addons as tfa
    
    return 



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import seaborn as sb
import os
import warnings
warnings.filterwarnings('ignore')
import tensorflow as tf
from tensorflow.keras.layers.experimental.preprocessing import RandomFlip
from tensorflow.keras.layers import DepthwiseConv2D,Layer
from tensorflow.keras.preprocessing import image_dataset_from_directory


import keras

from tensorflow.keras import backend as K
from tensorflow.keras.models import Sequential, Model,load_model
from tensorflow.keras.callbacks import EarlyStopping,ModelCheckpoint
from tensorflow.keras.layers import Dropout,ReLU,Layer, Input, add, Dense, Activation, ZeroPadding2D, BatchNormalization, Flatten, Conv2D, AveragePooling2D, MaxPooling2D, GlobalMaxPooling2D,MaxPool2D
from tensorflow.keras.preprocessing import image
from tensorflow.keras.initializers import glorot_uniform

# Model Checks
from sklearn.metrics import accuracy_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score
from sklearn.metrics import roc_auc_score


import matplotlib.pyplot as plt 
from sklearn.preprocessing import LabelBinarizer
from sklearn.metrics import roc_curve, auc, roc_auc_score
from itertools import cycle

#Model Graphs
from sklearn.metrics import confusion_matrix
from sklearn.metrics import ConfusionMatrixDisplay
from keras.layers import Activation, Lambda,GlobalAveragePooling2D,concatenate
import math
from tensorflow.keras.layers import DepthwiseConv2D
from keras.layers import SeparableConv2D,Add,GlobalAvgPool2D
from keras.layers import Resizing
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers
import tensorflow_addons as tfa


class conv2d_bn(tf.keras.layers.Layer):

    def __init__(self, filters, kernel_size, strides=1):
        super(conv2d_bn, self).__init__()
        self.conv = Conv2D(filters=filters,
                           kernel_size=kernel_size,
                           strides=strides,
                           padding='same',
                           use_bias=False)
        self.bn= BatchNormalization()

    def call(self, inputs):
        x = self.conv(inputs)
        x = self.bn(x)
        return x
    
    
#####################################################################

class sep_bn(tf.keras.layers.Layer):
    def __init__(self, filters, kernel_size, strides=1):
        super(sep_bn, self).__init__()
        self.SepConv2D = SeparableConv2D(filters=filters,
                                         kernel_size=kernel_size,
                                         strides=strides,
                                         padding='same',
                                         use_bias=False)
        self.bn= BatchNormalization()

    def call(self, inputs):
        x = self.SepConv2D(inputs)
        x = self.bn(x)
        return x
    
##################################################################


class EntryFlow(tf.keras.layers.Layer):
    def __init__(self):
        super(EntryFlow, self).__init__()
        self.conv2d_bn_0 = conv2d_bn(filters=32, kernel_size=3, strides=2)
        self.relu = ReLU()
        self.conv2d_bn_1 = conv2d_bn(filters=64, kernel_size=3)
        self.sep_bn_0 = sep_bn(filters=128, kernel_size=3)
        self.sep_bn_00 = sep_bn(filters=128, kernel_size=3)
        self.maxPool2D = MaxPool2D(pool_size=3, strides=2, padding='same')
        self.conv2d_bn_2 = conv2d_bn(filters=128, kernel_size=1, strides=2)
        self.sep_bn_1 = sep_bn(filters=256, kernel_size=3)
        self.sep_bn_11 = sep_bn(filters=256, kernel_size=3)
        self.sep_bn_2 = sep_bn(filters=728, kernel_size=3)
        self.sep_bn_22 = sep_bn(filters=728, kernel_size=3)
        self.conv2d_bn_3 = conv2d_bn(filters=256, kernel_size=1, strides=2)
        self.conv2d_bn_4 = conv2d_bn(filters=728, kernel_size=1, strides=2)
        
    def call(self, inputs):
        x = self.conv2d_bn_0(inputs)
        x = self.relu(x)
        x = self.conv2d_bn_1(x)
        x_res = self.relu(x) ##
        x = self.sep_bn_0(x_res)
        x = self.relu(x)
        x = self.sep_bn_00(x)
        x = self.maxPool2D(x)
        x_res = self.conv2d_bn_2(x_res)
        x = Add()([x_res, x])
        x = self.relu(x)
        x = self.sep_bn_1(x)
        x = self.relu(x)
        x = self.sep_bn_11(x)
        x = self.maxPool2D(x)
        x_res = self.conv2d_bn_3(x_res)
        x = Add()([x_res, x])
        x = self.relu(x)
        x = self.sep_bn_2(x)
        x = self.relu(x)
        x = self.sep_bn_22(x)
        x = self.maxPool2D(x)
        x_res = self.conv2d_bn_4(x_res)
        x = Add()([x_res, x])
        
        return x
    
#############################################################################

class MiddleFlow(tf.keras.layers.Layer):
    def __init__(self):
        super(MiddleFlow, self).__init__()
        self.relu = ReLU()
        self.sep_bn = sep_bn(filters=728, kernel_size=3)

    def call(self, inputs):
        for i in range(8):
            x = self.relu(inputs)
            x = self.sep_bn(x)
            x = self.relu(x)
            x = self.sep_bn(x)
            x = self.relu(x)
            x = self.sep_bn(x)
            inputs = Add()([inputs, x])
            
        return inputs
    
    
########################################################################


class ExitFlow(tf.keras.layers.Layer):
    def __init__(self, classes):
        super(ExitFlow, self).__init__()
        self.relu = ReLU()
        self.sep_bn_0 = sep_bn(filters=728, kernel_size=3)
        self.sep_bn_1 = sep_bn(filters=1024, kernel_size=3)
        self.maxPool2D = MaxPool2D(pool_size=3, strides=2, padding='same')
        self.conv2d_bn = conv2d_bn(filters=1024, kernel_size=1, strides=2)
        self.sep_bn_2 = sep_bn(filters=1536, kernel_size=3)
        self.sep_bn_3 = sep_bn(filters=2048, kernel_size=3)
        self.GAP2D = GlobalAvgPool2D()
        self.drop = Dropout(0.5)
        self.dense = Dense(classes, activation='softmax')

    def call(self, inputs):
        x = self.relu(inputs)
        x = self.sep_bn_0(x)
        x = self.relu(x)
        x = self.sep_bn_1(x)
        x = self.maxPool2D(x)
        
        inputs = self.conv2d_bn(inputs)
        x = Add()([inputs, x])
        x = self.sep_bn_2(x)
        x = self.relu(x)
        x = self.sep_bn_3(x)
        x = self.relu(x)
        x = self.GAP2D(x)
        x = self.drop(x)###
        x = self.dense(x)
            
        return x
    
    
########################################################################


def Xception(classes=1000):
    input_layer = Input(shape = [128, 128, 3])
    x = EntryFlow()(input_layer)
    x = MiddleFlow()(x)
    x = ExitFlow(classes)(x)
    
    return Model(input_layer, x)






import os
from setuptools import find_packages, setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="myPackageTFIMM",
    version="0.0.4",
    author="Alaa Elmor",
    author_email="alaa.m.elmor@gmail.com",
    description="image models",
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=["myPackageTFIMM"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)